package com.migao.controller;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.migao.config.authority.LoginUtils;
import com.migao.config.listener.ExcelListener;
import com.migao.config.response.BasePageRequest;
import com.migao.config.response.PageBeanOne;
import com.migao.config.response.ResponseBean;
import com.migao.config.response.ResponseBuilder;
import com.migao.entity.po.*;
import com.migao.entity.vo.req.*;
import com.migao.entity.vo.res.DemandLibraryPageQueryRes;
import com.migao.entity.vo.res.DemandLibraryQueryRes;
import com.migao.entity.vo.res.DictionaryListRes;
import com.migao.mapper.CompanyMapper;
import com.migao.mapper.SupplierMapper;
import com.migao.service.DemandLibraryService;
import com.migao.util.EntityUtils;
import com.migao.util.ProjectNoUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * @author dingsong
 */
@Api(tags = "007.需求库模快")
@RestController
@RequestMapping("/demandLibrary")
public class DemandLibraryController {
    @Resource
    DemandLibraryService demandLibraryService;
    @Resource
    CompanyMapper companyMapper;
    @Resource
    SupplierMapper supplierMapper;

    @ApiOperation("导入主需求")
    @PostMapping(value = "/insert")
    public ResponseBean<?> insert(@RequestBody DemandLibraryInsertReq demandLibrary){
       return demandLibraryService.insert(demandLibrary);
    }

    @ApiOperation("导入子需求")
    @PostMapping(value = "/insertSubProject")
    public ResponseBean<?> insertSubProject(@RequestBody DemandSubProjectReq demandSubProjectReq) {
        return demandLibraryService.insertSubProject(demandSubProjectReq);
    }

    @ApiOperation("删")
    @PostMapping(value = "/deleteById")
    public ResponseBean<?> deleteById(@RequestParam String id) {
        return demandLibraryService.deleteById(id);
    }

    @ApiOperation("更新需求")
    @PostMapping(value = "/update")
    public ResponseBean<?> update(@RequestBody DemandLibraryUpdateReq libraryUpdateReq) {
        return demandLibraryService.update(libraryUpdateReq);
    }

    @ApiOperation("分页查询所有需求数据")
    @PostMapping(value = "/pageQuery")
    public ResponseBean<PageBeanOne<DemandLibraryPageQueryRes>> pageQuery(@RequestBody BasePageRequest basePageRequest) {
        return demandLibraryService.pageQuery(basePageRequest);
    }

    @ApiOperation("分页查询当前用户需求数据")
    @PostMapping(value = "/pageQueryById")
    public ResponseBean<PageBeanOne<DemandLibraryPageQueryRes>> pageQueryById(@RequestBody BasePageRequest basePageRequest) {
        return demandLibraryService.pageQueryById(basePageRequest);
    }

    @ApiOperation("分页模糊查询所有需求数据")
    @PostMapping(value = "/pageQueryOne")
    public ResponseBean<PageBeanOne<DemandLibraryPageQueryRes>> pageQueryOne(@RequestBody DemandLibraryQueryReq demandLibraryQuery) {
        return demandLibraryService.pageQueryOne(demandLibraryQuery);
    }

    @ApiOperation("分页模糊查询当前用户需求数据")
    @PostMapping(value = "/pageQueryOneById")
    public ResponseBean<PageBeanOne<DemandLibraryPageQueryRes>> pageQueryOneById(@RequestBody DemandLibraryQueryReq demandLibraryQuery) {
        return demandLibraryService.pageQueryOneById(demandLibraryQuery);
    }

    @ApiOperation("查询单条需求数据")
    @PostMapping(value = "/findById")
    public ResponseBean<DemandLibraryQueryRes> findById(@RequestParam String id) {
        return demandLibraryService.findById(id);
    }

    @ApiOperation("关联项目")
    @PostMapping(value = "/findProject")
    public ResponseBean<List<Project>> findProject(@RequestParam String key) {
        return demandLibraryService.findProject(key);
    }

    @ApiOperation("需求分类下拉列表")
    @PostMapping(value = "/demandClassify")
    public ResponseBean<List<DictionaryListRes>> demandClassify() {
        return demandLibraryService.demandClassify();
    }

    @ApiOperation("需求状态下拉列表")
    @PostMapping(value = "/demandState")
    public ResponseBean<List<DictionaryListRes>> demandState() {
        return demandLibraryService.demandState();
    }

    @ApiOperation("需求单位下拉列表")
    @PostMapping(value = "/Company")
    public ResponseBean<List<Company>> Company() {
        return demandLibraryService.Company();
    }

    @ApiOperation("导入需求库信息")
    @PostMapping(value = "/importDemandLibrary")
    public ResponseBean importDemandLibrary(@RequestPart MultipartFile file) {
        //获取用户信息
        User user = LoginUtils.getUser();

        List<DemandLibraryExcelReq> demandState = new ArrayList<>();
        try {
            EasyExcel.read(file.getInputStream(), DemandLibraryExcelReq.class, new ExcelListener<>(demandState)).sheet().doRead();
        } catch (IOException e) {
            e.printStackTrace();
        }

        demandState.forEach(
                x -> {
                    DemandLibrary demandLibrary = EntityUtils.copyProperties(x,DemandLibrary.class);
                    //找到需求单位id
                    LambdaQueryWrapper<Company> company = new LambdaQueryWrapper<> ();
                    company.eq(Company::getOrgName,x.getUnitName());
                    Company companys = companyMapper.selectOne(company);
                    demandLibrary.setUnitNameId(companys.getId());
                    //找到公司id
                    LambdaQueryWrapper<Supplier> supplierWrapper = new LambdaQueryWrapper<> ();
                    supplierWrapper.eq(Supplier::getName,x.getImplementVender());
                    Supplier supplier = supplierMapper.selectOne(supplierWrapper);
                    demandLibrary.setImplementVenderId(supplier.getId());

                    demandLibrary.setDemandNumber(ProjectNoUtil.getCode());
                    demandLibrary.setAcceptDemandTime(Integer.parseInt(x.getAcceptDemandTime()));
                    demandLibrary.setPlanDemandTime(Integer.parseInt(x.getPlanDemandTime()));
                    demandLibrary.setSubmitTime(LocalDateTime.now());
                    demandLibrary.setDeleted(0);
                    demandLibrary.setCreateUserId(user.getId());
                    demandLibraryService.save(demandLibrary);
                }
        );

        return ResponseBuilder.ok(200,"导入成功");
    }

    @ApiOperation("导出需求库信息")
    @PostMapping(value = "/export")
    public ResponseBean export(@RequestParam String date) {
        return demandLibraryService.export(date);
    }
}
