package com.migao.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.migao.config.authority.LoginUtils;
import com.migao.config.response.*;
import com.migao.entity.po.DemandSchedule;
import com.migao.entity.po.User;
import com.migao.entity.vo.req.DemandScheduleInsertReq;
import com.migao.entity.vo.req.DemandScheduleQueryReq;
import com.migao.entity.vo.req.DemandScheduleUpdateReq;
import com.migao.entity.vo.res.DemandScheduleQueryRes;
import com.migao.mapper.DemandScheduleMapper;
import com.migao.service.DemandScheduleService;
import com.migao.util.EntityUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @author dingsong
 */
@Slf4j
@Service
@Transactional(rollbackFor = {Exception.class})
public class DemandScheduleServiceImpl extends ServiceImpl<DemandScheduleMapper, DemandSchedule> implements DemandScheduleService {
   @Resource
    DemandScheduleMapper demandScheduleMapper;


    @Override
    public ResponseBean<?> insert(DemandScheduleInsertReq dsi) {
        //获取当前登录用户的信息
        User user = LoginUtils.getUser();

        DemandSchedule demandSchedule = EntityUtils.copyProperties(dsi,DemandSchedule.class);
        demandSchedule.setSubmitTime(LocalDateTime.now());
        demandSchedule.setCreateUserId(user.getId());
        demandSchedule.setDeleted(0);
        demandScheduleMapper.insert(demandSchedule);

        return ResponseBuilder.ok(200,"新增成功");
    }

    @Override
    public ResponseBean<?> deleteById(String id) {
        Integer idInt = Integer.parseInt(id);
        DemandSchedule demandSchedule = demandScheduleMapper.selectById(idInt);
        demandSchedule.setDeleted(1);
        demandScheduleMapper.updateById(demandSchedule);
        return ResponseBuilder.ok(200,"删除成功");
    }

    @Override
    public ResponseBean<?> updateById(DemandScheduleUpdateReq updateReq) {
        DemandSchedule demandSchedule = EntityUtils.copyProperties(updateReq,DemandSchedule.class);
        demandSchedule.setSubmitTime(LocalDateTime.now());
        demandScheduleMapper.updateById(demandSchedule);
        return ResponseBuilder.ok(200,"更新成功");
    }

    @Override
    public ResponseBean<DemandSchedule> findById(String id) {
        Integer idInt = Integer.parseInt(id);
        DemandSchedule demandSchedule = demandScheduleMapper.selectById(idInt);

        return ResponseBuilder.ok(demandSchedule);
    }

    @Override
    public ResponseBean<PageBeanOne<DemandScheduleQueryRes>> pageQuery(BasePageRequest basePageRequest) {
        Integer page = basePageRequest.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = basePageRequest.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<DemandSchedule> dynamicQuery = new LambdaQueryWrapper<DemandSchedule> ();
        dynamicQuery.eq(DemandSchedule::getDeleted,0);
        dynamicQuery.orderByDesc(DemandSchedule::getSubmitTime);

        Page<DemandSchedule> all = demandScheduleMapper.selectPage(new Page<>(page, size),dynamicQuery);


        return ResponseBuilderOne.ok(
                all,
                element -> {
                    DemandScheduleQueryRes demandScheduleQueryRes = DemandScheduleQueryRes
                            .builder()
                            .id(element.getId())
                            .demandDescribe(element.getDemandDescribe())
                            .demandSchedule(element.getDemandSchedule())
                            .submitTime(element.getSubmitTime())
                            .build();
                    return demandScheduleQueryRes;
                  }
                );
    }

    @Override
    public ResponseBean<PageBeanOne<DemandScheduleQueryRes>> pageQueryById(BasePageRequest basePageRequest) {
        //获取用户信息
        User user = LoginUtils.getUser();

        Integer page = basePageRequest.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = basePageRequest.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<DemandSchedule> dynamicQuery = new LambdaQueryWrapper<DemandSchedule> ();
        dynamicQuery.eq(DemandSchedule::getDeleted,0);
        dynamicQuery.eq(DemandSchedule::getCreateUserId,user.getId());
        dynamicQuery.orderByDesc(DemandSchedule::getSubmitTime);

        Page<DemandSchedule> all = demandScheduleMapper.selectPage(new Page<>(page, size),dynamicQuery);


        return ResponseBuilderOne.ok(
                all,
                element -> {
                    DemandScheduleQueryRes demandScheduleQueryRes = DemandScheduleQueryRes
                            .builder()
                            .id(element.getId())
                            .demandDescribe(element.getDemandDescribe())
                            .demandSchedule(element.getDemandSchedule())
                            .submitTime(element.getSubmitTime())
                            .build();
                    return demandScheduleQueryRes;
                }
        );
    }

    @Override
    public ResponseBean<PageBeanOne<DemandScheduleQueryRes>> pageQueryOne(DemandScheduleQueryReq dsi) {
        Integer page = dsi.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = dsi.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<DemandSchedule> dynamicQuery = new LambdaQueryWrapper<DemandSchedule> ();
        dynamicQuery.eq(DemandSchedule::getDeleted,0);
        if(StringUtils.isNotBlank(dsi.getKey())){
            dynamicQuery.like(DemandSchedule::getDemandDescribe,dsi.getKey());
        }

        dynamicQuery.orderByDesc(DemandSchedule::getSubmitTime);

        Page<DemandSchedule> all = demandScheduleMapper.selectPage(new Page<>(page, size),dynamicQuery);


        return ResponseBuilderOne.ok(
                all,
                element -> {
                    DemandScheduleQueryRes demandScheduleQueryRes = DemandScheduleQueryRes
                            .builder()
                            .id(element.getId())
                            .demandDescribe(element.getDemandDescribe())
                            .demandSchedule(element.getDemandSchedule())
                            .submitTime(element.getSubmitTime())
                            .build();
                    return demandScheduleQueryRes;
                }
        );
    }

    @Override
    public ResponseBean<PageBeanOne<DemandScheduleQueryRes>> pageQueryOneById(DemandScheduleQueryReq dsi) {
        //获取用户信息
        User user = LoginUtils.getUser();

        Integer page = dsi.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = dsi.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<DemandSchedule> dynamicQuery = new LambdaQueryWrapper<DemandSchedule> ();
        dynamicQuery.eq(DemandSchedule::getDeleted,0);
        dynamicQuery.eq(DemandSchedule::getCreateUserId,user.getId());
        if(StringUtils.isNotBlank(dsi.getKey())){
            dynamicQuery.like(DemandSchedule::getDemandDescribe,dsi.getKey());
        }

        dynamicQuery.orderByDesc(DemandSchedule::getSubmitTime);

        Page<DemandSchedule> all = demandScheduleMapper.selectPage(new Page<>(page, size),dynamicQuery);


        return ResponseBuilderOne.ok(
                all,
                element -> {
                    DemandScheduleQueryRes demandScheduleQueryRes = DemandScheduleQueryRes
                            .builder()
                            .id(element.getId())
                            .demandDescribe(element.getDemandDescribe())
                            .demandSchedule(element.getDemandSchedule())
                            .submitTime(element.getSubmitTime())
                            .build();
                    return demandScheduleQueryRes;
                }
        );
    }


}
