package com.migao.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.migao.config.authority.LoginUtils;
import com.migao.config.constant.ProjectStatus;
import com.migao.config.constant.SystemConstant;
import com.migao.config.listener.ExcelListener;
import com.migao.config.properties.SystemProperties;
import com.migao.config.response.PageBean;
import com.migao.config.response.ResponseBean;
import com.migao.config.response.ResponseBuilder;
import com.migao.entity.po.FileInfo;
import com.migao.entity.po.Milestone;
import com.migao.entity.po.Project;
import com.migao.entity.vo.req.ImportMilestoneReq;
import com.migao.entity.vo.req.MilestoneInsertReq;
import com.migao.entity.vo.req.MilestonePageQueryReq;
import com.migao.entity.vo.req.MilestoneUpdateReq;
import com.migao.entity.vo.res.MilestoneQueryRes;
import com.migao.entity.vo.res.ProjectQueryRes;
import com.migao.mapper.FileInfoRepository;
import com.migao.mapper.MilestoneMapper;
import com.migao.mapper.MilestoneRepository;
import com.migao.mapper.ProjectMapper;
import com.migao.service.MilestoneService;
import com.migao.util.EntityUtils;
import com.migao.util.FileUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Slf4j
@Service
@Transactional(rollbackFor = {Exception.class})
public class MilestoneServiceImpl implements MilestoneService {


    @Resource
    private MilestoneMapper milestoneMapper;
    @Resource
    private MilestoneRepository milestoneRepository;
    @Resource
    private ProjectMapper projectMapper;
    @Resource
    private FileInfoRepository fileInfoRepository;
    @Resource
    private SystemProperties systemProperties;


    /**
     * 增
     *
     * @param milestoneInsertReq milestoneInsertReq
     * @return ?
     */
    @Override
    public ResponseBean<?> insert(MilestoneInsertReq milestoneInsertReq) {
        Milestone milestone = new Milestone();
        BeanUtils.copyProperties(milestoneInsertReq, milestone);
        milestone.setDeleted(0);
        milestone.setSchedule("0");
        milestone.setProjectId(milestoneInsertReq.getProjectId());
        if (LoginUtils.isLogin()){
            milestone.setUsername(LoginUtils.getUser().getUsername());
        }

        return ResponseBuilder.ok(milestoneMapper.insert(milestone));
    }

    /**
     * 分页查
     *
     * @param milestonePageQueryReq milestonePageQueryReq
     * @return ?
     */
    @Override
    public ResponseBean<PageBean<ProjectQueryRes>> pageQuery(MilestonePageQueryReq milestonePageQueryReq) {
        Integer page = milestonePageQueryReq.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = milestonePageQueryReq.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<Project> queryWrapper = new LambdaQueryWrapper<>();
        if (milestonePageQueryReq.getCreateTime() != null) {
        }
        if (StringUtils.isNotBlank(milestonePageQueryReq.getName())) {
            queryWrapper.eq(Project::getCreateTime, milestonePageQueryReq.getCreateTime());
        }
        if (StringUtils.isNotBlank(milestonePageQueryReq.getOrderStatus())) {
            queryWrapper.eq(Project::getOrderStatus, milestonePageQueryReq.getOrderStatus());
        }
        queryWrapper.ne(Project::getParentId, 0);
        //查询子项目并且采购状态是待提报
        queryWrapper.eq(Project::getPurchaseStatus, ProjectStatus.TO_BE_REPORTED);
        Page<Project> all = projectMapper.selectPage(new Page<>(page, size), queryWrapper);
        return ResponseBuilder.ok(
                all,
                element -> {
                    ProjectQueryRes projectQueryRes = ProjectQueryRes
                            .builder()
                            .id(element.getId())
                            .capital(String.valueOf(element.getCapital()))
                            .content(element.getContent())
                            .cost(String.valueOf(element.getCost()))
                            .createTime(element.getCreateTime())
                            .level(element.getLevel())
                            .name(element.getName())
                            .progress(element.getProgress())
                            .projectClassify(element.getProjectClassify())
                            .specialtyClassify(element.getSpecialtyClassify())
                            .total(String.valueOf(element.getTotal()))
                            .totalInvestment(String.valueOf(element.getTotalInvestment()))
                            .twoClassify(element.getTwoClassify())
                            .unit(element.getUnit())
                            .orderStatus(String.valueOf(element.getOrderStatus()))
                            .reserveStatus(String.valueOf(element.getReserveStatus()))
                            .build();
                    return projectQueryRes;
                }
        );
    }

    @Override
    public ResponseBean<?> importData(Integer fileId,Integer projectId) {
        List<ImportMilestoneReq> list = new ArrayList<>();
        Optional<FileInfo> byId = fileInfoRepository.findById(fileId);
        byId.ifPresent(
                fileInfo -> {
                    EasyExcel.read(
                            fileInfo.getDiskPath(),
                            ImportMilestoneReq.class,
                            new ExcelListener<>(list)
                    ).sheet().doRead();
                }
        );
        log.info("读取结果:{}", list.size());
        list.forEach(
                data->{
                    Milestone milestone = EntityUtils.copyProperties(data, Milestone.class);

                    //绑定到项目
                    Project project = projectMapper.selectById(projectId);
                    if (project!=null){
                        milestone.setProjectId(project.getId());
                    }
                    milestone.setDeleted(0);
                    milestoneMapper.insert(milestone);

                }
        );
        return ResponseBuilder.ok();
    }

    @Override
    public ResponseBean<?> export(MilestonePageQueryReq milestonePageQueryReq) {
        List<ProjectQueryRes> data = pageQuery(milestonePageQueryReq).getData().getRow();
        String fileName = StringUtils.join(
                FileUtils.getFileNo(), SystemConstant.EXCEL_SUFFIX
        );
        String filePath = StringUtils.join(
                systemProperties.getFileStoreDirPath(), fileName
        );
        String fileUrl = StringUtils.join(
                systemProperties.getDownloadPrefix(), fileName
        );
        ExcelWriter excelWriter = EasyExcel.write(filePath, ProjectQueryRes.class).build();
        WriteSheet writeSheet = EasyExcel.writerSheet("里程碑数据").build();
        excelWriter.write(data, writeSheet);
        //关闭流
        excelWriter.finish();
        return ResponseBuilder.ok(fileUrl);
    }

    @Override
    public ResponseBean<ProjectQueryRes> findById(Integer id) {
        Project project = projectMapper.selectById(id);
        ProjectQueryRes projectQueryRes = new ProjectQueryRes();
        BeanUtils.copyProperties(project, projectQueryRes);
        return ResponseBuilder.ok(projectQueryRes);
    }

    @Override
    public ResponseBean<List<MilestoneQueryRes>> selectMilestone(Integer id) {
        LambdaQueryWrapper<Milestone> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Milestone::getProjectId, id);
        List<Milestone> all = milestoneMapper.selectList(queryWrapper);
        List<MilestoneQueryRes> list = new ArrayList<>();
        for (Milestone milestone : all) {
            MilestoneQueryRes res = EntityUtils.copyProperties(milestone, MilestoneQueryRes.class);
            list.add(res);
        }
        return ResponseBuilder.ok(list);
    }

    @Override
    public ResponseBean<?> template() {
        String filePath = StringUtils.join(
                systemProperties.getDownloadPrefix() +
                        systemProperties.getFileTemplateDir() + "/里程碑导入模板.xlsx"
        );
        return ResponseBuilder.ok(filePath);
    }

    /**
     * 删
     *
     * @param id id
     * @return ?
     */
    @Override
    public ResponseBean<?> deleteById(Integer id) {
        milestoneRepository.findById(id).ifPresent(
                i -> {
                    i.setDeleted(1);
                    milestoneRepository.save(i);
                }
        );
        return ResponseBuilder.ok();
    }

    /**
     * 批量删
     *
     * @param ids ids
     * @return ?
     */
    @Override
    public ResponseBean<?> deleteBatch(List<Integer> ids) {
        milestoneRepository.findAllById(ids).forEach(
                i -> {
                    i.setDeleted(1);
                    milestoneRepository.save(i);
                }
        );
        return ResponseBuilder.ok();
    }

    /**
     * 改
     *
     * @param milestoneUpdateReq milestoneUpdateReq
     * @return ?
     */
    @Override
    public ResponseBean<?> update(MilestoneUpdateReq milestoneUpdateReq) {
        milestoneRepository.save(EntityUtils.copyProperties(milestoneUpdateReq, Milestone.class));
        return ResponseBuilder.ok();
    }

}
